#light

open System
open System.IO
open System.Net

// ----------------------------------------------------------------------------
// HTTP 

let getPage (url:string) = 
  try
    let req = WebRequest.Create(url)
    let resp = req.GetResponse()
    let stream = resp.GetResponseStream()
    let reader = new IO.StreamReader(stream)
    reader.ReadToEnd()
  with e ->
    e.ToString()
  
getPage "http://localhost/quickstartv20"
getPage "http://localhost/quickstartv20/default.aspx"


// ----------------------------------------------------------------------------
// Regular expressions

open System.Text.RegularExpressions

let url = "http://localhost/quickstartv20/default.aspx"
let def = getPage url
let regUrl = new Regex("href=\"(?<url>[^\"]*)\"")
let matches = regUrl.Matches(def)

let regBaseUrl = new Regex("(?<base>http://[^\\?]*)/[^/\\?]*\\??.*")
let urlMatch = regBaseUrl.Match(url)
let base = (urlMatch.Groups.get_Item("base")).Value

// ----------------------------------------------------------------------------
// URL Filtering

let (links:list<Match>) = IEnumerable.untyped_to_list matches
let links1 = links |> List.map ( fun m -> (m.Groups.get_Item("url")).Value )
let links2 = links1 |> List.filter ( fun s -> (not (s.StartsWith("http:"))) && (not (s.StartsWith("."))) )
let links3 = links2 |> List.map ( fun l -> base + "/" + l )

// ----------------------------------------------------------------------------
// Same code, but wrapped inside function

let getAllLinks url =
  let def = getPage url
  let matches = regUrl.Matches(def)
  let urlMatch = regBaseUrl.Match(url)
  let base = (urlMatch.Groups.get_Item("base")).Value
  let i = base.IndexOf("/", 10)
  let serverBase = (if (i = -1) then base else base.Substring(0, i))
  
  let (links:list<Match>) = IEnumerable.untyped_to_list matches
  links 
    |> List.map ( fun m -> (m.Groups.get_Item("url")).Value )
    |> List.filter ( fun s -> (not (s.StartsWith("mailto:"))) && (not (s.StartsWith("http:"))) && (not (s.StartsWith("."))) )
    |> List.map ( fun l -> 
      if (l.StartsWith("/")) then 
          serverBase + l
        else 
          base + "/" + l )
    |> List.map ( fun l -> 
      let l2 = l.ToLower()
      let i = l2.IndexOf("#")
      if (i = -1) then l2 else l2.Substring(0, i) )

// DEMO:
// getAllLinks "http://localhost/quickstartv20/default.aspx"
    
// ----------------------------------------------------------------------------
// Save links to table

open System.Collections.Generic

let dict = new Dictionary<String, List<String> >()
let addUrls root links =
  let urls = 
    (if (dict.ContainsKey(root)) then 
       dict.[root] 
     else 
       let u = new List<String>()
       dict.Add(root, u)
       u )
  
  let i = urls.Count
  links |> List.iter ( fun url ->
    if (not (urls.Contains(url))) then urls.Add(url) )
    
  (urls.Count - i)

// DEMO:
// addUrls "http://localhost/quickstartv20/default.aspx" (getAllLinks "http://localhost/quickstartv20/default.aspx")

// ----------------------------------------------------------------------------
// Walk through the whole web

let rec walk depth url =
  let links = (getAllLinks url)
  let n = addUrls url links
  System.Console.WriteLine("{0}/{1}, {2}", n, links.Length, url)
  if ((n > 0) && (depth > 0)) then links |> List.iter ( fun l -> walk (depth - 1) l )


// DEMO:
// walk 2 url

// DEMO:
// dict.Count

// ----------------------------------------------------------------------------
// 

// Save result to file
let wr = new StreamWriter("C:\\temp.html")
dict |> IEnumerable.iter ( fun pair ->
    wr.WriteLine("<h1><a name=\"{1}\"></a>{0}</h1>", pair.Key, pair.Key.Replace("\"", "_") )
    wr.WriteLine("<ul>")
    pair.Value |> IEnumerable.iter ( fun url -> 
        if (dict.ContainsKey(url)) then
          wr.WriteLine("<li><a href=\"#{1}\">{0}</a>", url, url.Replace("\"", "_") ) 
        else
          wr.WriteLine("<li>{0}", url) 
        wr.WriteLine(" [<a href=\"{0}\">go</a>]</li>", url);
      )
    wr.WriteLine("</ul>")
  )
wr.Close()  